"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createUseCssAndCx = exports.createCssAndCx = void 0;
const classnames_1 = require("./tools/classnames");
const serialize_1 = require("@emotion/serialize");
const utils_1 = require("@emotion/utils");
const useGuaranteedMemo_1 = require("./tools/useGuaranteedMemo");
//import { matchCSSObject } from "./types";
exports.createCssAndCx = (() => {
    function merge(registered, css, className) {
        const registeredStyles = [];
        const rawClassName = (0, utils_1.getRegisteredStyles)(registered, registeredStyles, className);
        if (registeredStyles.length < 2) {
            return className;
        }
        return rawClassName + css(registeredStyles);
    }
    function createCssAndCx(params) {
        const { cache } = params;
        const css = (...args) => {
            const serialized = (0, serialize_1.serializeStyles)(args, cache.registered);
            (0, utils_1.insertStyles)(cache, serialized, false);
            const className = `${cache.key}-${serialized.name}`;
            /*
            scope: {
                const arg = args[0];

                if (!matchCSSObject(arg)) {
                    break scope;
                }

                increaseSpecificityToTakePrecedenceOverMediaQuerries.saveClassNameCSSObjectMapping(
                    cache,
                    className,
                    arg,
                );
            }
            */
            return className;
        };
        const cx = (...args) => {
            const className = (0, classnames_1.classnames)(args);
            /*
            const feat27FixedClassnames =
                increaseSpecificityToTakePrecedenceOverMediaQuerries.fixClassName(
                    cache,
                    className,
                    css,
                );
            */
            //return merge(cache.registered, css, feat27FixedClassnames);
            return merge(cache.registered, css, className);
        };
        return { css, cx };
    }
    return { createCssAndCx };
})().createCssAndCx;
function createUseCssAndCx(params) {
    const { useCache } = params;
    function useCssAndCx() {
        const cache = useCache();
        const { css, cx } = (0, useGuaranteedMemo_1.useGuaranteedMemo)(() => (0, exports.createCssAndCx)({ cache }), [cache]);
        return { css, cx };
    }
    return { useCssAndCx };
}
exports.createUseCssAndCx = createUseCssAndCx;
//// https://github.com/garronej/tss-react/issues/27
//const increaseSpecificityToTakePrecedenceOverMediaQuerries = (() => {
//    const cssObjectMapByCache = new WeakMap<
//        EmotionCache,
//        Map<string, CSSObject>
//    >();
//
//    return {
//        "saveClassNameCSSObjectMapping": (
//            cache: EmotionCache,
//            className: string,
//            cssObject: CSSObject,
//        ) => {
//            let cssObjectMap = cssObjectMapByCache.get(cache);
//
//            if (cssObjectMap === undefined) {
//                cssObjectMap = new Map();
//                cssObjectMapByCache.set(cache, cssObjectMap);
//            }
//
//            cssObjectMap.set(className, cssObject);
//        },
//        "fixClassName": (() => {
//            function fix(
//                classNameCSSObjects: [
//                    string /*className*/,
//                    CSSObject | undefined,
//                ][],
//            ): (string | CSSObject)[] {
//                let isThereAnyMediaQueriesInPreviousClasses = false;
//
//                return classNameCSSObjects.map(([className, cssObject]) => {
//                    if (cssObject === undefined) {
//                        return className;
//                    }
//
//                    let out: string | CSSObject;
//
//                    if (!isThereAnyMediaQueriesInPreviousClasses) {
//                        out = className;
//
//                        for (const key in cssObject) {
//                            if (key.startsWith("@media")) {
//                                isThereAnyMediaQueriesInPreviousClasses = true;
//                                break;
//                            }
//                        }
//                    } else {
//                        out = {
//                            "&&": cssObject,
//                        };
//                    }
//
//                    return out;
//                });
//            }
//
//            return (
//                cache: EmotionCache,
//                className: string,
//                css: Css,
//            ): string => {
//                const cssObjectMap = cssObjectMapByCache.get(cache);
//
//                return classnames(
//                    fix(
//                        className
//                            .split(" ")
//                            .map(className => [
//                                className,
//                                cssObjectMap?.get(className),
//                            ]),
//                    ).map(classNameOrCSSObject =>
//                        typeof classNameOrCSSObject === "string"
//                            ? classNameOrCSSObject
//                            : css(classNameOrCSSObject),
//                    ),
//                );
//            };
//        })(),
//    };
//})();
